function f=mufll(p,d)
% F = MUFLL(P,D)
% P = [A,B0,phi,lambda,xi,sigma)
% D = data (time)
% generates asym(p(B)) for fit of isotropic flux lattice with fft algorithm
% A = Asymmetry
% B0 = field (G)
% phi = phase (degrees)
% lambda = London penetration depth (nm) 
% xi= coherence length (nm)
% sigma = convoluted gaussian width (mus-1)
% d(1,:) is the time vector in musec 
%
% f is asymmetry, time shifted to start as cos(wt+phi) at t=0
%
% N=8 : 2^N * 2^N is the number of reciprocal vectors (size of G lattice) 
% db, the field vector b resolution over which p(b) is produced, is
% calculated from the frequency resolution required by the time
% bins, dt
% calculates with
% fft2 = Two-dimensional fast Fourier transform (Matlab)
%
% direct lattice a=a[i]; b=a[i+jsqrt(3)]/2
% reciprocal lattice a*=2pi/a[i-j/sqrt(3)];  b*=2pi/a[2j/sqrt(3)]
% the fourier component of the field, bG for the reciprocal vector G, is:
% bG=B0*exp[-(xi*G)^2/2/(1-Bc2/B0)]/[1+(lambda*G)^2/(1-Bc2/B0)]
%
% the square modulus of the vector G(h,k)=h.a* + k.b* is
% G^2=(ha* + k b*)dot(h a* + k b*) 
%    = (h 2pi/a)^2 (1+1/3) + (k 2pi/a)^2 4/3 + 2hk (-2/3)
%    = 4/3 (2pi/a)^2 (h^2 + k^2 - h.k)
%
% RDR Aug 2009
% (original simulations in matlab/flux/simasym.m)

global MU_TABLE

% all calculations in SI units
fi0=2.068e-15; % flux quantum in  Tm^2
B0=p(2)*1e-4; % converted to T
lambda=p(4)*1e-9; % converted to m
xi=p(5)*1e-9; % converted to m
a=sqrt(4*fi0/sqrt(3)/B0); % triangular lattice spacing (equilateral)
     %Bc2=fi0/(xi^2*2*pi); % upper critical field (unused)
     %x=B0/Bc2;      % x = reduced field  B0/Bc2 (unused)
astar=2*pi/a;  % astar = factor for G vectors
N=8; % base for lattice grid seems a reasonble compromise between
     % speed and accuracy
[h,k]=meshgrid(-2^(N):2^(N)-1,-2^(N):2^(N)-1);
fac=2/sqrt(3);% triangular factor (ratio of side to height)
% corrected London
% bg=B0*exp(-(xi*astar*fac)^2/2/(1-x)*(h.^2+k.^2-h.*k))./(1+(lambda*astar*fac)^2*(h.^2+k.^2-h.*k)/(1-x));
%
% FOURIER COMPONENT OF FLL FIELDS 
% p(2)=B0, p(3)=phi, p(4)=lambda, p(5)=xi, pure London
bg=B0*exp(-(xi*astar*fac)^2/2*(h.^2+k.^2-h.*k))./...
   (1+(lambda*astar*fac)^2*(h.^2+k.^2-h.*k));
%
BMAX=sum(sum(bg)); % core value (London)
BSAD=real(sum(sum(bg.*exp(-i*pi*h)))); % saddle value 
BMIN=real(sum(sum(bg.*exp(-i*2*pi*(h/3+k/3))))); % minimal value
br=fft2(fftshift(bg)); % direct space fields
[m,n]=size(br); % lattice sise
Br=reshape(real(br),m*n,1); % reshape to an array to calculate histogram
                            % below
% before, prepare for FFTs
t0=d(1,1); %  will be generated as a time shift
dt=d(1,2)-d(1,1); % to determines dnu
Nt=size(d,2);
m=fix(log(Nt)/log(2));
if 2^m==Nt % if time array length is a power of 2
    N=Nt;
else
    N=2^(m+1); % find next power of 2
end
n=[0:N -N+1:-1]; % generate fft independent variable
nn=[-N+1:N]; % monotonic
dnu=1/(2*N*dt);
%dw=dnu*2*pi;
db=dnu/MU_TABLE.GAMU/10000; % field resolution dnu=1/(2*N*dt) db=dnu/gamu
bb=nn*db; % monotonic field for the histogram
b=n*db; % field for the ifft
% HISTOGRAM AND HAND-MADE HILBERT TRANSFORM
ppdf=hist(Br,bb); % positive frequencies only
prdf=[ppdf(N:2*N) ppdf(2*N-1:-1:N+1)]; % real part mirror positive to
                                       % negative frequencies
%im=min(find(b>BMAX)); % for debugging
%plot([b(1:im) b(end-im+1:end)],[prdf(1:im) prdf(end-im+1:end)])
pidf=[ppdf(N:2*N) -ppdf(2*N-1:-1:N+1)]; % imag part inverts mirror
norm=sum(prdf);
prdf=prdf/norm; % unitary area 
pidf=pidf/norm; % same
pdf=cos(p(3)/180*pi)*prdf+i*sin(-p(3)/180*pi)*pidf; % full FFT of
                                                   % p(b), all components
                                  % starting as cos(wt+phi) 
pdf=pdf.*exp(i*(t0*pi/N/dt*n)); % time shift, with
                            % dw=2*pi*n/(2*N*dt);
f=real(2*N*ifft(pdf)); % Inverse FFT
f=p(1)*f(1:Nt).*exp(-((d(1,:)*p(6)).^2)/2); % cut to original
                                             % size, muliply by asymmetry 
                                             % and convolute the FFT
                                             % with a gaussian
                                             % p(5)=sigma
